@@ -1,9 +1,12 @@
 import sys
 from threading import Thread
 
+from cartopy import crs
 import time
 import holoviews as hv
 import panel as pn
+import cartopy.crs as ccrs
+
 import pandas as pd
 import hvplot.pandas
 import hvplot.xarray
@@ -19,6 +22,22 @@ hv.extension("bokeh")
 hv.Store.renderers
 
 
+def get_coastline(ds, lon_name, lat_name):
+    lon_max = ds[lon_name].values.max()
+    lon_min = ds[lon_name].values.min()
+
+    lat_max = ds[lat_name].values.max()
+    lat_min = ds[lat_name].values.min()
+
+    if  ((lon_max - lon_min) * (lat_max - lat_min))/(360**2) < 0.0016:
+        return '10m'
+
+    if  ((lon_max - lon_min) * (lat_max - lat_min))/(360**2) < 0.25:
+        return '50m'
+
+    return '110m'
+
+
 def change_coords(dx):
 
     ds = dx.copy()
@@ -65,6 +84,8 @@ def change_coords(dx):
 def is_curvilinear(ds):
 
     ds_dims = get_dims(ds)
+    if ds_dims.longitude.values[0] is None or ds_dims.latitude.values[0] is None:
+        return False
 
     if len(ds.coords[ds_dims.longitude.values[0]].dims) > 1:
         return True
@@ -82,6 +103,9 @@ def in_notebook():
     Returns ``True`` if the module is running in IPython kernel,
     ``False`` if in IPython shell or other Python shell.
     """
+    if "spyder" in sys.modules:
+        return False
+
     return "ipykernel" in sys.modules
 
 
@@ -92,8 +116,9 @@ def ncplot(x, vars=None):
     -------------
     x : object or str
         xarray object or file path
-    vars: list or str
+    vars : list or str
         Variables you want to plot. Everything will be plotted if this is not supplied
+
     """
 
     log = False
@@ -113,6 +138,7 @@ def ncplot(x, vars=None):
         ds = x
 
     quadmesh = False
+
     if is_curvilinear(ds):
         ds = change_coords(ds)
         quadmesh = True
@@ -139,6 +165,18 @@ def ncplot(x, vars=None):
 
     time_name = df_dims.time[0]
 
+    # time name maybe cannot be parsed. If "time" is among the coords, use that
+
+    if time_name is None:
+        candidates = list()
+        n_candidates = 0
+        for x in list(ds.coords):
+            if x.startswith("time"):
+                candidates.append(x)
+                n_candidates += 0
+        if len(candidates) == 1:
+            time_name = candidates[0]
+
     if time_name is None:
         n_times = 0
     else:
@@ -164,10 +202,16 @@ def ncplot(x, vars=None):
 
     # code below figures out what is a variable, not a coordinate. Could be improved...
 
+    coord_list = list(ds.coords)
+
+    for cc in coord_list:
+        if len(ds[cc].values.ravel()) <= 1:
+            if cc in list(ds.dims):
+                ds = ds.squeeze(cc, drop=True)
+
     if type(vars) is list:
         new_vars = []
         for vv in vars:
-            # if vv == "lon_bnds":
             set_coords = set(
                 [x for x in list(ds.coords) if "vertice" not in x and "bnds" not in x]
             )
@@ -179,6 +223,7 @@ def ncplot(x, vars=None):
         if len(vars) == 1:
             vars = vars[0]
 
+
     if type(vars) is list:
         for vv in vars:
             if vv not in list(ds.variables):
@@ -192,14 +237,12 @@ def ncplot(x, vars=None):
 
     # get rid of coordinates without multiple values
 
-
-
     coord_list = list(ds.coords)
 
     for cc in coord_list:
         if len(ds[cc].values.ravel()) <= 1:
             if cc in list(ds.dims):
-                ds = ds.squeeze(cc, drop = True)
+                ds = ds.squeeze(cc, drop=True)
 
     coord_list = list(ds.coords)
 
@@ -209,6 +252,7 @@ def ncplot(x, vars=None):
 
     # line plot
 
+
     if len([x for x in coord_df.length if x > 1]) == 1:
 
         df = ds.to_dataframe().reset_index()
@@ -270,9 +314,18 @@ def ncplot(x, vars=None):
         selection = [x for x in df.columns if x in vars or x == x_var or x == y_var]
 
         df = df.loc[:, selection].melt([x_var, y_var]).drop_duplicates()
-        if (len(ds.coords[lon_name].values) == 1) or (
-            len(ds.coords[lat_name].values) == 1
-        ):
+
+        # we need to make sure this is not 2d geographic data
+        case1 = 0
+        if lon_name is not None and lon_name in list(ds.coords):
+            if len(ds.coords[lon_name].values) > 1:
+                case1 += 1
+
+        if lat_name is not None and lat_name in list(ds.coords):
+            if len(ds.coords[lat_name].values) > 1:
+                case1 += 1
+
+        if case1 <= 1:
 
             if type(vars) is list:
                 intplot = df.drop_duplicates().hvplot.heatmap(
@@ -330,21 +383,30 @@ def ncplot(x, vars=None):
         non_map = True
 
         if lon_name is not None and lat_name is not None:
-            lons = int(coord_df.query("coord == @lon_name").length)
-            lats = int(coord_df.query("coord == @lat_name").length)
+            if (lon_name is not None) and (lon_name in list(ds.coords)):
+                lons = int(coord_df.query("coord == @lon_name").length)
+            else:
+                lons = 0
+
+            if (lat_name) is not None and (lat_name in list(ds.coords)):
+                lats = int(coord_df.query("coord == @lat_name").length)
+            else:
+                lats = 0
+
             if lons > 1 and lats > 1:
                 non_map = False
 
         time_in = False
 
         possible = 0
+
         for x in coord_list:
             if "time" in x:
                 time_in = True
                 possible += 1
 
-        if possible > 1:
-            time_in = False
+        # if possible > 1:
+        #    time_in = False
 
         if time_name in coord_list and time_in and non_map:
 
@@ -373,9 +435,16 @@ def ncplot(x, vars=None):
 
                 df = df.loc[:, selection]
                 df = df.melt([x_var, y_var, time_name]).drop_duplicates()
-                if (len(ds.coords[lon_name].values) == 1) or (
-                    len(ds.coords[lat_name].values) == 1
-                ):
+                case1 = 0
+                if lon_name is not None and lon_name in list(ds.coords):
+                    if len(ds.coords[lon_name].values) > 1:
+                        case1 += 1
+
+                if lat_name is not None and lat_name in list(ds.coords):
+                    if len(ds.coords[lat_name].values) > 1:
+                        case1 += 1
+
+                if case1 <= 1:
 
                     intplot = df.drop_duplicates().hvplot.heatmap(
                         x=x_var,
@@ -473,6 +542,8 @@ def ncplot(x, vars=None):
 
         if is_curvilinear(ds):
 
+            coastline = get_coastline(ds, lon_name, lat_name)
+
             intplot = ds.hvplot.quadmesh(
                 lon_name,
                 lat_name,
@@ -480,10 +551,16 @@ def ncplot(x, vars=None):
                 dynamic=True,
                 cmap="viridis",
                 logz=log,
+                coastline = coastline,projection=ccrs.PlateCarree(),
+                rasterize = True,
                 responsive=in_notebook() is False,
             )
+            # intplot = pn.Column(pn.WidgetBox(w1), intplot)
         else:
 
+            # w1 = pn.widgets.Select(name='Coastline', options=[True, False])
+
+            coastline = get_coastline(ds, lon_name, lat_name)
             intplot = ds.hvplot.image(
                 lon_name,
                 lat_name,
@@ -491,8 +568,11 @@ def ncplot(x, vars=None):
                 dynamic=True,
                 cmap="viridis",
                 logz=log,
+                coastline = coastline,
+                projection=ccrs.PlateCarree(),
                 responsive=in_notebook() is False,
             )
+            # intplot = pn.Column(pn.WidgetBox(w1), intplot)
 
         if in_notebook():
             return intplot
@@ -516,6 +596,8 @@ def ncplot(x, vars=None):
 
         if (self_max.values > 0) and (self_min.values < 0):
             if is_curvilinear(ds):
+                # w1 = pn.widgets.Select(name='Coastline', options=[True, False])
+                coastline = get_coastline(ds, lon_name, lat_name)
                 intplot = ds.hvplot.quadmesh(
                     lon_name,
                     lat_name,
@@ -523,18 +605,25 @@ def ncplot(x, vars=None):
                     dynamic=True,
                     logz=log,
                     cmap="RdBu_r",
+                    rasterize = True,
                     responsive=(in_notebook() is False),
                 ).redim.range(**{vars: (-v_max, v_max)})
+                # intplot = pn.Row(pn.WidgetBox(w1), intplot)
             else:
+                # w1 = pn.widgets.Select(name='Coastline', options=[True, False])
+                coastline = get_coastline(ds, lon_name, lat_name)
                 intplot = ds.hvplot.image(
                     lon_name,
                     lat_name,
                     vars,
                     dynamic=True,
                     logz=log,
+                    coastline = coastline,
+                    projection=ccrs.PlateCarree(),
                     cmap="RdBu_r",
                     responsive=(in_notebook() is False),
                 ).redim.range(**{vars: (-v_max, v_max)})
+                # intplot = pn.Row(pn.WidgetBox(w1), intplot)
 
             if in_notebook():
                 return intplot
@@ -555,10 +644,12 @@ def ncplot(x, vars=None):
                     dynamic=True,
                     logz=log,
                     cmap="viridis",
+                    rasterize = True,
                     responsive=(in_notebook() is False),
                 ).redim.range(**{vars: (self_min.values, v_max)})
             else:
 
+                coastline = get_coastline(ds, lon_name, lat_name)
                 intplot = ds.hvplot.image(
                     lon_name,
                     lat_name,
@@ -566,6 +657,7 @@ def ncplot(x, vars=None):
                     dynamic=True,
                     logz=log,
                     cmap="viridis",
+                    coastline = coastline,
                     responsive=(in_notebook() is False),
                 ).redim.range(**{vars: (self_min.values, v_max)})
 
